<?php

if (!defined('XOONIPS2WEKO_PATH')) {
	define('XOONIPS2WEKO_PATH', XOOPS_ROOT_PATH . '/modules/xoonips2weko');
}

include_once XOONIPS2WEKO_PATH . "/include/functions.php";
include_once "ItemTypeInterface.php";

class Xoonips2weko_ItemTypeArticle implements ItemTypeInterface
{
	private $item_type_id;

	private $metadata_values = array(
		"item_key",
		"id",
		"title",
		"title_kana",
		"title_romaji",
		"sub_title_name",
		"sub_title_kana",
		"sub_title_romaji",
		"author_id",
		"author_name",
		"author_kana",
		"author_romaji",
		"author_affiliation",
		"author_affiliation_translation",
		"author_role",
		"author_link",
		"edition",
		"publish_place",
		"publisher",
		"publisher_kana",
		"publisher_romaji",
		"year_f",
		"year_t",
		"date_create",
		"date_create_normalize",
		"date_update",
		"date_update_normalize",
		"date_record",
		"date_record_normalize",
		"physical_descriptions",
		"jtitle",
		"jtitle_translation",
		"jtitle_volume",
		"jtitle_issue",
		"dateofissued",
		"dateofissued_normalize",
		"jtitle_year",
		"jtitle_month",
		"jtitle_spage",
		"jtitle_epage",
		"id_issns",
		"id_isbns",
		"id_dois",
		"id_uris",
		"self_doi_ra",
		"self_doi",
		"naid",
		"ichushi",
		"id_locals",
		"grant_id",
		"date_of_granted",
		"date_of_granted_normalize",
		"degree_name",
		"grantor",
		"abstract",
		"table_of_contents",
		"keywords",
		"ndc_classifications",
		"description",
		"langs",
		"type_of_resource",
		"genre",
		"textversion",
		"file",
		"mime_type",
		"preview",
		"uris",
		"access_condition",
		"last_update_date",
		"last_update_date_normalize",
		"creation_date",
		"creation_date_normalize",
		"user",
		"log",
		"index",
		"related",
		"views",
		"downloaded",
		"weko_item_name"
	);

	function __construct()
	{
		$this->item_type_id = get_item_type_id("article");
	}

	public function metadataValid(&$metadata)
	{
		$ret = true;
		foreach ($metadata as $key => $value) {
			if (!in_array($key, $this->metadata_values)) {
				add_log("ERROR metadata $key is invalid\n");
				$ret = false;
			}
		}

		return $ret;
	}

	public function exportItemMetadata($item_id, &$metadata)
	{
		# item_basic
		$ret = get_item_basic($item_id, $basic);
		if (!$ret) {
			add_log("ERROR exportItemMetadata false\n");
			return false;
		}
		if ($basic['item_type_id'] != $this->item_type_id)
			return 2;

		if (isset($metadata['item_key']))
			$metadata['item_key'] = $item_id;

		if (isset($metadata['id']))
			$metadata['id'] = $basic['doi'];

		if (isset($metadata['description']))
			$metadata['description'] = $basic['description'];

		if (isset($metadata['user']))
			$metadata['user'] = get_user_name($basic['uid']);

		if (isset($metadata['last_update_date']))
			$metadata['last_update_date'] = date('M j, Y H:i:s', $basic['last_update_date']);

                if (isset($metadata['last_update_date_normalize']))
                        $metadata['last_update_date_normalize'] = date('Y/m/d', $basic['last_update_date']);

		if (isset($metadata['creation_date']))
			$metadata['creation_date'] = date('M j, Y H:i:s', $basic['creation_date']);

		if (isset($metadata['creation_date_normalize']))
			$metadata['creation_date_normalize'] = date('Y/m/d', $basic['creation_date']);

		# item_detail

		$ret = get_item_details($item_id, $details);
		if (!$ret) {
			add_log("ERROR exportItemMetadata get_item_details false\n");
			return false;
		}

		foreach ($details as $key => $value) {
			if (isset($metadata[$key]))
				$metadata[$key] = $value;
		}

		if (isset($metadata['date_create_normalize']) && preg_match('/^[0-9]{4}-[0-9]{2}-[0-9]{2}$/', $details['date_create'])) {
			$metadata['date_create_normalize'] = str_replace('-', '/', $details['date_create']);
		}

                if (isset($metadata['date_update_normalize']) && preg_match('/^[0-9]{4}-[0-9]{2}-[0-9]{2}$/', $details['date_update'])) {
			$metadata['date_update_normalize'] = str_replace('-', '/', $details['date_update']);
                }

                if (isset($metadata['date_record_normalize']) && preg_match('/^[0-9]{4}-[0-9]{2}-[0-9]{2}$/', $details['date_record'])) {
			$metadata['date_record_normalize'] = str_replace('-', '/', $details['date_record']);
                }

                if (isset($metadata['date_of_granted_normalize']) && preg_match('/^[0-9]{4}-[0-9]{2}-[0-9]{2}$/', $details['date_of_granted'])) {
			$metadata['date_of_granted_normalize'] = str_replace('-', '/', $details['date_of_granted']);
                }

		if (isset($metadata['dateofissued'])) {
			$ret = make_date_iso8601($details['jtitle_year'], $details['jtitle_month'], 0);
			if ($ret)
				$metadata['dateofissued'] = $ret;
		}

                if (isset($metadata['dateofissued_normalize'])) {
			$dateofissued_normalize = "";
			if (!empty($details['jtitle_year']) && preg_match('/^[0-9]{4}$/', $details['jtitle_year']) 
			&& !empty($details['jtitle_month']) && preg_match('/^[0-9]{1,2}$/', $details['jtitle_month'])) {
				$dateofissued_normalize = sprintf("%04d", $details['jtitle_year']) . "/" . sprintf("%02d", $details['jtitle_month']) . "/01";
			} else if (!empty($details['jtitle_year']) && preg_match('/^[0-9]{4}$/', $details['jtitle_year'])) {
				$dateofissued_normalize = sprintf("%04d", $details['jtitle_year']) . "/01/01";
			}
			$metadata['dateofissued_normalize'] = $dateofissued_normalize;
                }

		// selfDOI ra option
                if (isset($metadata['self_doi_ra'])) {
			$ra = "";
			if (!empty($details['self_doi'])) {
				$self_dois = array();
				$self_dois = explode("#", $details['self_doi']);
				if (!empty($self_dois[1])) {
					if (in_array($self_dois[0], array('JaLC', 'CrossRef'))) {
						$ra = $self_dois[0];
					}
					$metadata['self_doi'] = $self_dois[1];
				} else {
					$metadata['self_doi'] = $self_dois[0];
				}
				if (empty($ra)) {
					if (in_array(_MD_XNPARTICLE_OAIPMH_JUNII2_RA_OPTION, array('JaLC', 'CrossRef'))) {
						$ra = _MD_XNPARTICLE_OAIPMH_JUNII2_RA_OPTION;
					}
				}
				$metadata['self_doi_ra'] = $ra;
			}
		}

		# item_detail_child_sub_title

		$ret = get_item_child_sub_titles($item_id, $subtitles);
		if (!$ret) {
			add_log("ERROR exportItemMetadata get_item_child_sub_title false\n");
			return false;
		}

		foreach ($subtitles as $key => $value) {
			if (isset($metadata[$key]))
				$metadata[$key] = $value;
		}

		# item_detail_child_author

		$ret = get_item_child_authors($item_id, $authors);
		if (!$ret) {
			add_log("ERROR exportItemMetadata get_item_child_authors false\n");
			return false;
		}

		foreach ($authors as $key => $value) {
			if (isset($metadata[$key]))
				$metadata[$key] = $value;
		}

		# item_detail_child_keywords

		if (isset($metadata['keywords'])) {
			$ret = get_item_child_keywords($item_id, $keywords);
			if (!$ret) {
				add_log("ERROR exportItemMetadata get_item_child_keywords false\n");
				return false;
			}
			if (isset($keywords['keywords']))
				$metadata['keywords'] = $keywords['keywords'];
		}

		# item_detail_child_ndc_classifications

		if (isset($metadata['ndc_classifications'])) {
			$ret = get_item_child_ndc_classifications($item_id, $ndc);
			if (!$ret) {
				add_log("ERROR exportItemMetadata get_item_child_ndc_classifications false\n");
				return false;
			}
			if (isset($ndc['ndc_classifications']))
				$metadata['ndc_classifications'] = $ndc['ndc_classifications'];
		}

		# item_detail_child_physical_descriptions

		if (isset($metadata['physical_descriptions'])) {
			$ret = get_item_child_physical_descriptions($item_id, $formats);
			if (!$ret) {
				add_log("ERROR exportItemMetadata get_item_child_physical_descriptions false\n");
				return false;
			}
			if (isset($formats['physical_descriptions']))
				$metadata['physical_descriptions'] = $formats['physical_descriptions'];
		}

		# item_detail_child_langs

		if (isset($metadata['langs'])) {
			$ret = get_item_child_langs($item_id, $languages);
			if (!$ret) {
				add_log("ERROR exportItemMetadata get_item_child_langs false\n");
				return false;
			}
			if (isset($languages['langs']))
				$metadata['langs'] = $languages['langs'];
		}

		# item_detail_child_id_issns

		if (isset($metadata['id_issns'])) {
			$ret = get_item_child_issns($item_id, $issns);
			if (!$ret) {
				add_log("ERROR exportItemMetadata get_item_child_issns false\n");
				return false;
			}
			if (isset($issns['id_issns']))
				$metadata['id_issns'] = $issns['id_issns'];
		}

		# item_detail_child_id_isbns

		if (isset($metadata['id_isbns'])) {
			$ret = get_item_child_isbns($item_id, $isbns);
			if (!$ret) {
				add_log("ERROR exportItemMetadata get_item_child_isbns false\n");
				return false;
			}
			if (isset($isbns['id_isbns']))
				$metadata['id_isbns'] = $isbns['id_isbns'];
		}

		# item_detail_child_id_dois

		if (isset($metadata['id_dois'])) {
			$ret = get_item_child_dois($item_id, $dois);
			if (!$ret) {
				add_log("ERROR exportItemMetadata get_item_child_dois false\n");
				return false;
			}
			if (isset($dois['id_dois']))
				$metadata['id_dois'] = $dois['id_dois'];
		}

		# item_detail_child_id_uris

		if (isset($metadata['id_uris'])) {
			$ret = get_item_child_id_uris($item_id, $id_uris);
			if (!$ret) {
				add_log("ERROR exportItemMetadata get_item_child_id_uris false\n");
				return false;
			}
			if (isset($id_uris['id_uris']))
				$metadata['id_uris'] = $id_uris['id_uris'];
		}

		# item_detail_child_id_locals

		if (isset($metadata['id_locals'])) {
			$ret = get_item_child_locals($item_id, $locals);
			if (!$ret) {
				add_log("ERROR exportItemMetadata get_item_child_locals false\n");
				return false;
			}
			if (isset($locals['id_locals']))
				$metadata['id_locals'] = $locals['id_locals'];
		}

		# item_detail_child_uris

		if (isset($metadata['uris'])) {
			$ret = get_item_child_uris($item_id, $uris);
			if (!$ret) {
				add_log("ERROR exportItemMetadata get_item_child_uris false\n");
				return false;
			}
			if (isset($uris['uris']))
				$metadata['uris'] = $uris['uris'];
		}

		# xoonips_ranking_viewed_item

		if (isset($metadata['views']))
			$metadata['views'] = get_ranking_viewed_item($item_id);

		# xoonips_ranking_downloaded_item

		if (isset($metadata['downloaded']))
			$metadata['downloaded'] = get_ranking_downloaded_item($item_id);

		# index

		if (isset($metadata['index'])) {
			$ret = get_item_indexes($item_id, $indexes);
			if (!$ret) {
				add_log("ERROR exportItemMetadata get_item_indexes false\n");
				return false;
			}
			if (isset($indexes['index']))
				$metadata['index'] = $indexes['index'];
		}

		# xoonips_changelog

		if (isset($metadata['log'])) {
			$ret = get_item_change_log($item_id, $changelog);
			if (!$ret) {
				add_log("ERROR exportItemMetadata get_item_change_log false\n");
				return false;
			}
			if (isset($changelog['log']))
				$metadata['log'] = $changelog['log'];
		}

		# xoonips_related_to

		if (isset($metadata['related'])) {
			$ret = get_item_related($item_id, $related);
			if (!$ret) {
				add_log("ERROR exportItemMetadata get_item_related false\n");
				return false;
			}
			if (isset($related['related']))
				$metadata['related'] = $related['related'];
		}

		# weko_item_name
		if (isset($metadata['weko_item_name'])) {
			$metadata['weko_item_name'] = get_weko_item_name($details['genre']);
		}

		return true;
	}

	public function exportItemFile($item_id, $item_dir, &$metadata)
	{
		global $delimiter;

		$file = get_file_info($item_id);
		if (!$file)
			return 2;

		$export_dir = "$item_dir/$item_id";
		if (!mkdir($export_dir)) {
			add_log("ERROR can not make directory $export_dir\n");
			return false;
		}

		foreach ($file as $f) {
			$same_file = false;
			list($file_id, $file_type_name, $original_file_name, $file_size, $mime_type, $thumbnail_file, $caption) = $f;
			$copy_from = xnpGetUploadFilePath($file_id);
			if (!file_exists($copy_from)) {
				add_log("Warning no exists file $copy_from\n");
				continue;
			}

			$fname = "";
			if (empty($caption))
				$fname = windows_sanitize($original_file_name);
			else
				$fname = windows_sanitize($caption);

			$copy_to = "$export_dir/" . mb_convert_encoding($fname, "UTF-8", "auto");

			if (file_exists($copy_to)) {
				if (is_same_file($copy_from, $copy_to)){
					$same_file = true;
				}else{
					$fextension = pathinfo($fname, PATHINFO_EXTENSION );
					if (!$fextension) {
						$fbase = $fname;
					} else {
						$fextension = ".".$fextension;
						$fbase = basename($fname, $fextension);
					}						
					
					for ($i = 1; file_exists($copy_to); $i++) {
						if (!$fextension) {
							$fname = sprintf('%s_%03s', $fbase, $i);
						} else {
							$fname = sprintf('%s_%03s%s', $fbase, $i, $fextension);
						}
						$fname = windows_sanitize($fname);
						$copy_to = $export_dir.'/'.mb_convert_encoding($fname, "UTF-8", "auto");
					}
				}
			}
			if (!$same_file && !@copy($copy_from, $copy_to)) {
				add_log("Warning can not copy from $copy_from\n");
				return false;
			}

			if ($file_type_name === "article_attachment") {
				if (isset($metadata['file'])) {
					if (empty($metadata['file']))
						$metadata['file'] = $fname;
					else
						$metadata['file'] .= $delimiter . $fname;
				}
				if (isset($metadata['mime_type'])) {
					if (empty($metadata['mime_type']))
						$metadata['mime_type'] = $mime_type;
					else
						$metadata['mime_type'] .= $delimiter . $mime_type;
				}
			} else {
				if (isset($metadata['preview'])) {
					if (empty($metadata['preview']))
						$metadata['preview'] = $fname;
					else
						$metadata['preview'] .= $delimiter . $fname;
				}
			}
			add_log("file copy $copy_from -> $copy_to\n");
		}
		return true;
	}
}

