<?php

if (!defined('XOONIPS2WEKO_PATH')) {
	define('XOONIPS2WEKO_PATH', XOOPS_ROOT_PATH . '/modules/xoonips2weko');
}

include_once XOONIPS2WEKO_PATH . "/include/functions.php";
include_once "ItemTypeInterface.php";

class Xoonips2weko_ItemTypeBinder implements ItemTypeInterface
{
	private $item_type_id;

	private $item_type = "Data or Dataset";

	private $metadata_values = array(
		"item_key",
		"id",
		"title",
		"keywords",
		"description",
		"last_update_date",
		"last_update_date_normalize",
		"creation_date",
		"creation_date_normalize",
		"user",
		"item_type",
		"log",
		"index",
		"extra",
		"binder_item_id",
		"views",
		"weko_item_name"
	);

	function __construct()
	{
		$this->item_type_id = get_item_type_id("binder");
	}

	public function metadataValid(&$metadata)
	{
		$ret = true;
		foreach ($metadata as $key => $value) {
			if (!in_array($key, $this->metadata_values)) {
				add_log("ERROR metadata $key is invalid\n");
				$ret = false;
			}
		}

		return $ret;
	}

	public function exportItemMetadata($item_id, &$metadata)
	{
		# item_basic
		$ret = get_item_basic($item_id, $basic);
		if (!$ret) {
			add_log("ERROR exportItemMetadata false\n");
			return false;
		}
		if ($basic['item_type_id'] != $this->item_type_id)
			return 2;

		if (isset($metadata['item_key']))
			$metadata['item_key'] = $item_id;

		if (isset($metadata['id']))
			$metadata['id'] = $basic['doi'];

		if (isset($metadata['description']))
			$metadata['description'] = $basic['description'];

		if (isset($metadata['last_update_date']))
			$metadata['last_update_date'] = date('M j, Y H:i:s', $basic['last_update_date']);

		if (isset($metadata['last_update_date_normalize']))
			$metadata['last_update_date_normalize'] = date('Y/m/d', $basic['last_update_date']);

		if (isset($metadata['creationa_date']))
			$metadata['creation_date'] = date('M j, Y H:i:s', $basic['creation_date']);

		if (isset($metadata['creation_date_normalize']))
			$metadata['creation_date_normalize'] = date('Y/m/d', $basic['creation_date']);

		if (isset($metadata['user']))
			$metadata['user'] = get_user_name($basic['uid']);

		if (isset($metadata['item_type']))
			$metadata['item_type'] = $this->item_type;

		# item_title

		if (isset($metadata['title'])) {
			$ret = get_item_title($item_id);
			if ($ret === false) {
				add_log("ERROR exportItemMetadata get_item_title false\n");
				return false;
			}
			$metadata['title'] = $ret;
		}

		# item_keyword

		if (isset($metadata['keywords'])) {
			$ret = get_item_keywords($item_id);
			if ($ret === false) {
				add_log("ERROR exportItemMetadata get_item_keywords false\n");
				return false;
			}
			$metadata['keywords'] = $ret;
		}

		# xnpbinder_item_detail

		if (isset($metadata['extra'])) {
			$detail = $this->getBinderItemDetail($item_id);
			if ($detail === false) {
				add_log("ERROR exportItemMetadata getBinderItemDetail false\n");
				return false;
			}
			if (isset($detail['extra']))
				$metadata['extra'] = $detail['extra'];
		}

		# index

		if (isset($metadata['index'])) {
			$ret = get_item_indexes($item_id, $indexes);
			if (!$ret) {
				add_log("ERROR exportItemMetadata get_item_indexes false\n");
				return false;
			}
			$metadata['index'] = $indexes['index'];
		}

		# xoonips_changelog

		if (isset($metadata['log'])) {
			$ret = get_item_change_log($item_id, $changelog);
			if (!$ret) {
				add_log("ERROR exportItemMetadata get_item_change_log false\n");
				return false;
			}
			$metadata['log'] = $changelog['log'];
		}

		# xnpbinder_binder_item_link

		if (isset($metadata['binder_item_id'])) {
			$ids = $this->getBinderItemLink($item_id);
			if (empty($ids)) {
				$metadata['binder_item_id'] = "";
			} else{
				global $delimiter;
				$metadata['binder_item_id'] = make_tsv_record_str($ids, $delimiter);
			}
		}
		
		# xoonips_ranking_viewed_item

		if (isset($metadata['views'])) {
			$metadata['views'] = get_ranking_viewed_item($item_id);
		}

		# weko_item_name
                if (isset($metadata['weko_item_name'])) {
                        $metadata['weko_item_name'] = get_weko_item_name($this->item_type);
                }

		return true;
	}

	public function exportItemFile($item_id, $item_dir, &$metadata)
	{
		# Nothing to do
		return true;
	}

	private function getBinderItemDetail($item_id)
	{
		global $xoopsDB;
		$itemDetailTable = $xoopsDB->prefix("xnpbinder_item_detail");
		$sql = "SELECT extra FROM $itemDetailTable WHERE binder_id = $item_id";

		$result = $xoopsDB->query($sql);
		if (!$result) {
			add_log("ERROR getBinderItemDetail false\n");
			return false;
		}

		$details = array();
		while ($row = $xoopsDB->fetchArray($result)) {
			$details[] = $row;
		}
		return $details;
	}

	private function getBinderItemLink($item_id)
	{
		global $xoopsDB;
		$itemLinkTable = $xoopsDB->prefix("xnpbinder_binder_item_link");
		$sql = "SELECT item_id FROM $itemLinkTable WHERE binder_id = $item_id";

		$result = $xoopsDB->query($sql);
		if (!$result) {
			add_log("ERROR getBinderItemLink false\n");
			return false;
		}

		$links = array();
		while ($row = $xoopsDB->fetchArray($result)) {
			$links[] = $row['item_id'];
		}
		return $links;
	}
}

