﻿<?php
/* XooNIps2WEKO Module
 * class/ItemTypeConference.class.php
 * $Revision: 1.10
 */

include_once XOONIPS2WEKO_PATH . "/include/functions.php";
include_once "ItemTypeInterface.php";

class Xoonips2weko_ItemTypeConference implements ItemTypeInterface
{
	private $item_type_id;

        private $item_type = "Presentation";

	private $metadata_values = array(
		"item_key",
		"id",
		"langs",
		"conference_title",
		"place",
		"conference_from_year",
		"conference_from_month",
		"conference_from_mday",
		"temporal",
		"conference_to_year",
		"conference_to_month",
		"conference_to_mday",
		"title",
		"author",
		"last_update_date",
		"last_update_date_normalize",
		"creation_date",
		"creation_date_normalize",
		"user",
		"item_type",
		"log",
		"abstract",
		"conference_file",
		"conference_file_mime_type",
		"presentation_type",
		"conference_paper",
		"conference_paper_mime_type",
		"index",
		"related",
		"views",
		"downloaded",
                "weko_item_name"
		);
	
	function __construct()
	{
		$this->item_type_id = get_item_type_id("Conference");
	}
	
	public function metadataValid(&$metadata)
	{
		$ret = true;
		foreach ($metadata as $key => $value) {
			if (!in_array($key, $this->metadata_values)) {
				add_log("ERROR metadata $key is invalid\n");
				$ret = false;
			}
		}

		return $ret;
	}
	
	public function exportItemMetadata($item_id, &$metadata)
	{
		// item_key
		if (isset($metadata['item_key'])) {
			$metadata['item_key'] = $item_id;
		}
		// item_basic
		$ret = get_item_basic($item_id, $basic);
		if (!$ret) {
			add_log("ERROR  export_item_metadata false\n");
			return false;
		} else {
			if ($basic['item_type_id'] != $this->item_type_id) {
				return 2;
			} else {
				// id
				if (isset($metadata['id'])) {
					$metadata['id'] = $basic['doi'];
				}
				
				// langs
				if (isset($metadata['langs'])) {
					$metadata['langs'] = $basic['lang'];
				}
				
				// last_update_date
				if (isset($metadata['last_update_date'])) {
					$metadata['last_update_date'] = date('M j, Y H:i:s', $basic['last_update_date']);
				}

                                // last_update_date_normalize
                                if (isset($metadata['last_update_date_normalize'])) {
                                        $metadata['last_update_date_normalize'] = date('Y/m/d', $basic['last_update_date']);
                                }
				
				// creation_date
				if (isset($metadata['creation_date'])) {
					$metadata['creation_date'] = date('M j, Y H:i:s', $basic['creation_date']);
				}

                                // creation_date_normalize
                                if (isset($metadata['creation_date_normalize'])) {
                                        $metadata['creation_date_normalize'] = date('Y/m/d', $basic['creation_date']);
                                }
				
				// user
				$user_name = get_user_name($basic['uid']);
				if (isset($metadata['user'])) {
					$metadata['user'] = $user_name;
				}
			}
		}
		
		// item_type
		if (isset($metadata['item_type'])) {
			$metadata['item_type'] = $this->item_type;
		}
		
		// xnpconference_item_detail
		$ret = $this->get_conference_item_details($item_id, $conference_details);
		if (!$ret) {
			add_log("ERROR exportItemMetadata get_conference_item_details false\n");
            return false;
        } else {
			foreach ( $conference_details as $key => $value ) {
				if (isset($metadata[$key])) {
					$metadata[$key] = $value;
				}
			}
        }
		
		// temporal
		if (isset($metadata['temporal'])) {
			$from_date = make_date_iso8601($conference_details['conference_from_year'], 
											$conference_details['conference_from_month'] , $conference_details['conference_from_mday']);
			$to_date = make_date_iso8601($conference_details['conference_to_year'],
											$conference_details['conference_to_month'] , $conference_details['conference_to_mday']);
			if ($from_date && $to_date) {
				if ($from_date == $to_date) {
					$metadata['temporal'] = $from_date;
				} else {
					$metadata['temporal'] = $from_date . "/" . $to_date;
				}
			}
		}
		
		// item_title
		if (isset($metadata['title'])) {
			$title = get_item_title($item_id);
			if ($title === false) {
				add_log("ERROR exportItemMetadata get_item_title false\n");
				return false;

			} else {
				$metadata['title'] = $title;
			}
		}
		
		// index
		if (isset($metadata['index'])) {
			$ret = get_item_indexes($item_id, $indexes);
			if (!$ret) {
				add_log("ERROR exportItemMetadata get_item_indexes false\n");
				return false;
			} elseif (isset($indexes['index'])) {
				$metadata['index'] = $indexes['index'];
			}
		}
		
		// xnpconference_author
		if (isset($metadata['author'])) {
			$authors = $this->get_conference_authors($item_id);
			if ($authors === false) {
				add_log("ERROR exportItemMetadata get_conference_authors false\n");
				return false;
			} else {
				$metadata['author'] = $authors;
			}
		}
		
		// xoonips_changelog
		if (isset($metadata['log'])) {
			$ret = get_item_change_log($item_id, $log);
			if (!$ret) {
				add_log("ERROR exportItemMetadata get_item_change_log false\n");
				return false;
			} elseif (isset($log['log'])) {
				$metadata['log'] = $log['log'];
			}
		}
		
		// related_to
		if (isset($metadata['related'])) {
			$ret = get_item_related($item_id, $related);
			if (!$ret) {
				add_log("ERROR exportItemMetadata get_item_related false\n");
				return false;
			} elseif (isset($related['related'])) {
				$metadata['related'] = $related['related'];
			}
		}
		
		// xoonips_ranking_viewed_item
		if (isset($metadata['views'])) {
			$metadata['views'] = get_ranking_viewed_item($item_id);
		}

		// xoonips_ranking_downloaded_item
		if (isset($metadata['downloaded'])) {
			$metadata['downloaded'] = get_ranking_downloaded_item($item_id);
		}

                // weko_item_name
                if (isset($metadata['weko_item_name'])) {
                        $metadata['weko_item_name'] = get_weko_item_name($this->item_type);
                }


		return true;
	}
	
	public function exportItemFile($item_id, $item_dir, &$metadata)
	{
		global $delimiter;
		$fname = "";
		$file = get_file_info($item_id);
		if (!$file) {
			return 2;
		}
		
		$export_file_dir = $item_dir."/".$item_id;
        if ( !mkdir( $export_file_dir, 0755 ) ) {
			add_log("ERROR can not make directory ".$export_file_dir."\n");
			return false;
		} else {
			foreach ($file as $f) {
				$same_file = false;
				list( $file_id, $file_type_name, $original_file_name, $file_size, $mime_type, $thumbnail_file, $caption) = $f;
				$copy_from = xnpGetUploadFilePath($file_id);
				if ( file_exists($copy_from) ) {
					$fname = windows_sanitize($original_file_name);
					$export_file_path = $export_file_dir.'/'.mb_convert_encoding($fname, "UTF-8", "auto");
					
					if (file_exists($export_file_path)) {
						if (is_same_file($copy_from, $export_file_path)){
							$same_file = true;
						}else{
							$fextension = pathinfo($fname, PATHINFO_EXTENSION );
							if (!$fextension) {
								$fbase = $fname;
							} else {
								$fextension = ".".$fextension;
								$fbase = basename($fname, $fextension);
							}						
							
							for ($i = 1; file_exists($export_file_path); $i++) {
								if (!$fextension) {
									$fname = sprintf('%s_%03s', $fbase, $i);
								} else {
									$fname = sprintf('%s_%03s%s', $fbase, $i, $fextension);
								}
								$fname = windows_sanitize($fname);
								$export_file_path = $export_file_dir.'/'.mb_convert_encoding($fname, "UTF-8", "auto");
							}
						}
					}
					
					if (!$same_file && !@copy( $copy_from, $export_file_path)) {
						add_log("Warning can not copy from ".$copy_from."\n");
						return false;
					} else {
						if ($file_type_name == "conference_file") {
							if (isset($metadata['conference_file'])) {
								$metadata['conference_file'] = $fname;
							}
							if (isset($metadata['conference_file_mime_type'])) {
								$metadata['conference_file_mime_type'] = $mime_type;
							}
						} elseif ($file_type_name == "conference_paper") {
							if (isset($metadata['conference_paper'])) {
								$metadata['conference_paper'] = $fname;
							}
							if (isset($metadata['conference_paper_mime_type'])) {
								$metadata['conference_paper_mime_type'] = $mime_type;
							}
						} else {
							// 無視
						}
						add_log("file copy `".$copy_from."' -> `" . $export_file_path . "'\n");
					}
				} else {
					add_log("Warning no exists file " . $copy_from . "\n");
				}
			}
		}
		
		return true;
	}
	
	private function get_conference_item_details($item_id, &$conference_details)
	{
	    global $xoopsDB;
        $sql = "SELECT conference_title, place, conference_from_year, conference_from_month, conference_from_mday,";
		$sql .= " conference_to_year, conference_to_month, conference_to_mday, abstract, presentation_type";
		$sql .= " FROM " . $xoopsDB->prefix( "xnpconference_item_detail" );
        $sql .= " WHERE conference_id = " . $item_id;
        $result = $xoopsDB->query( $sql );
        if ( !$result ) {
			add_log("ERROR get_conference_item_details false ".$sql."\n");
            return false;
        } else {
			while ( $row = $xoopsDB->fetchArray( $result ) ) {
				$conference_details = $row;
			}
        }
        return true;
	}
	
	private function get_conference_authors($item_id)
	{
		$authors = "";
		global $xoopsDB;
		global $delimiter;
		$sql = "SELECT author FROM " . $xoopsDB->prefix( "xnpconference_author" );
		$sql .= " WHERE conference_id = " . $item_id;
		$result = $xoopsDB->query( $sql );
		if ( !$result ) {
			add_log("ERROR get_conference_authors false ".$sql."\n");
			return false;
		} else {
			while ( $row = $xoopsDB->fetchArray( $result ) ) {
				if (empty($authors)) {
					$authors = $row['author'];
				} else {
					$authors .= $delimiter . $row['author'];
				}
			}
		}
		return $authors;
	}
}

