﻿<?php
/* XooNIps2WEKO Module
 * class/ItemTypeStimulus.class.php
 * $Revision: 1.10
 */

include_once XOONIPS2WEKO_PATH . "/include/functions.php";
include_once "ItemTypeInterface.php";

class Xoonips2weko_ItemTypeStimulus implements ItemTypeInterface
{
	private $item_type_id;
	
        private $item_type = "Data or Dataset";

	private	$metadata_values = array(
		"item_key",
		"id",
		"langs",
		"title",
		"keywords",
		"description",
		"publication_year",
		"publication_month",
		"publication_mday",
		"publication_date",
		"publication_date_normalize",
		"temporal",
		"last_update_date",
		"last_update_date_normalize",
		"creation_date",
		"creation_date_normalize",
		"user",
		"item_type",
		"log",
		"stimulus_type",
		"developer",
		"preview",
		"file",
		"mime_type",
		"readme",
		"rights",
		"index",
		"related",
		"views",
		"downloaded",
                "weko_item_name"
	);
	
	function __construct()
	{
		$this->item_type_id = get_item_type_id("Stimulus");
	}
	
	public function metadataValid(&$metadata)
	{
		$ret = true;
		foreach ($metadata as $key => $value) {
			if (!in_array($key, $this->metadata_values)) {
				add_log("ERROR metadata $key is invalid\n");
				$ret = false;
			}
		}

		return $ret;
	}
	
	public function exportItemMetadata($item_id, &$metadata)
	{
		// item_key
		if (isset($metadata['item_key'])) {
			$metadata['item_key'] = $item_id;
		}
		// item_basic
		$ret = get_item_basic($item_id, $basic);
		if (!$ret) {
			add_log("ERROR  export_item_metadata false\n");
			return false;
		} else {
			if ($basic['item_type_id'] != $this->item_type_id) {
				return 2;
			} else {
				// id
				if (isset($metadata['id'])) {
					$metadata['id'] = $basic['doi'];
				}
				
				// langs
				if (isset($metadata['langs'])) {
					$metadata['langs'] = $basic['lang'];
				}
				
				// description
				if (isset($metadata['description'])) {
					$metadata['description'] = $basic['description'];
				}
				
				// publication_year
				if (isset($metadata['publication_year'])) {
					$metadata['publication_year'] = $basic['publication_year'];
				}
				
				// publication_month
				if (isset($metadata['publication_month'])) {
					$metadata['publication_month'] = $basic['publication_month'];
				}
				
				// publication_mday
				if (isset($metadata['publication_mday'])) {
					$metadata['publication_mday'] = $basic['publication_mday'];
				}

                                // publication_date
                                if (isset($metadata['publication_date'])) {
                                        if (!empty($basic['publication_year']) && !empty($basic['publication_month']) && !empty($basic['publication_mday'])) {
                                                $ret = make_date_iso8601(sprintf("%04d", $basic['publication_year']), sprintf("%02d", $basic['publication_month']), sprintf("%02d", $basic['publication_mday']));
                                        } else if (!empty($basic['publication_year']) && !empty($basic['publication_month'])) {
                                                $ret = make_date_iso8601(sprintf("%04d", $basic['publication_year']), sprintf("%02d", $basic['publication_month']), 0);
                                        } else {
                                                $ret = make_date_iso8601(sprintf("%04d", $basic['publication_year']), 0, 0);
                                        }
                                        if ($ret)
                                                $metadata['publication_date'] = $ret;
                                }

                                // publication_date_normalize
                                if (isset($metadata['publication_date_normalize'])) {
                                        $ret = "";
                                        if (!empty($basic['publication_year']) && !empty($basic['publication_month']) && !empty($basic['publication_mday'])) {
                                                $ret = sprintf("%04d", $basic['publication_year']) . "/" . sprintf("%02d", $basic['publication_month']) . "/" . sprintf("%02d", $basic['publication_mday']);
                                        } else if (!empty($basic['publication_year']) && !empty($basic['publication_month'])) {
                                                $ret = sprintf("%04d", $basic['publication_year']) . "/" . sprintf("%02d", $basic['publication_month']) . "/01";
                                        } else {
                                                $ret = sprintf("%04d", $basic['publication_year']) . "/01/01";
                                        }
                                        if ($ret)
                                                $metadata['publication_date_normalize'] = $ret;
                                }
				
				// temporal
				if (isset($metadata['temporal'])) {
					$ret = make_date_iso8601($basic['publication_year'],
								 $basic['publication_month'], $basic['publication_mday']);
					if ($ret)
						$metadata['temporal'] = $ret;
				}
				
				// last_update_date
				if (isset($metadata['last_update_date'])) {
					$metadata['last_update_date'] = date('M j, Y H:i:s', $basic['last_update_date']);
				}

                                // last_update_date_normalize
                                if (isset($metadata['last_update_date_normalize'])) {
                                        $metadata['last_update_date_normalize'] = date('Y/m/d', $basic['last_update_date']);
                                }
				
				// creation_date
				if (isset($metadata['creation_date'])) {
					$metadata['creation_date'] = date('M j, Y H:i:s', $basic['creation_date']);
				}
				
                                // creation_date_normalize
                                if (isset($metadata['creation_date_normalize'])) {
                                        $metadata['creation_date_normalize'] = date('Y/m/d', $basic['creation_date']);
                                }

				// user
				$user_name = get_user_name($basic['uid']);
				if (isset($metadata['user'])) {
					$metadata['user'] = $user_name;
				}
			}
		}
		
		// item_type
		if (isset($metadata['item_type'])) {
			$metadata['item_type'] = $this->item_type;
		} 
		
		// title
		if (isset($metadata['title'])) {
			$title = get_item_title($item_id);
			if ($title === false) {
				add_log("ERROR exportItemMetadata get_item_title false\n");
				return false;
			} else {
				$metadata['title'] = $title;
			}
		}

		// item_keyword
		if (isset($metadata['keywords'])) {
			$keywords = get_item_keywords($item_id);
			if ($keywords === false) {
				add_log("ERROR exportItemMetadata get_item_keywords false\n");
				return false;
			} else {
				$metadata['keywords'] = $keywords;
			}
		}
		
		// xnpstimulus_item_detail
		$detail = $this->getStimulusItemDetail($item_id);
		if (!$detail) {
			add_log("ERROR exportItemMetadata getStimulusItemDetail false\n");
			return false;
		} else {
			foreach ( $detail as $key => $value ) {
				if (isset($metadata[$key])) {
					$metadata[$key] = $value;
				}
			}
		}
		
		// xnpstimulus_developer
		if (isset($metadata['developer'])) {
			$developer = $this->getStimulusDeveloper($item_id);
			if ($developer === false) {
				add_log("ERROR exportItemMetadata getStimulusDeveloper false\n");
				return false;
			} else {
				$metadata['developer'] = $developer;
			}
		}
		
		// index
		if (isset($metadata['index'])) {
			$ret = get_item_indexes($item_id, $indexes);
			if (!$ret) {
				add_log("ERROR exportItemMetadata get_item_indexes false\n");
				return false;
			} elseif (isset($indexes['index'])) {
				$metadata['index'] = $indexes['index'];
			}
		}

		// xoonips_changelog
		if (isset($metadata['log'])) {
			$ret = get_item_change_log($item_id, $changelog);
			if (!$ret) {
				add_log("ERROR exportItemMetadata get_item_change_log false\n");
				return false;
			} elseif (isset($changelog['log'])) {
				$metadata['log'] = $changelog['log'];
			}
		}
		
		// related_to
		if (isset($metadata['related'])) {
			$ret = get_item_related($item_id, $related);
			if (!$ret) {
				add_log("ERROR exportItemMetadata get_item_related false\n");
				return false;
			} elseif (isset($related['related'])) {
				$metadata['related'] = $related['related'];
			}
		}
		
		// xoonips_ranking_viewed_item
		if (isset($metadata['views'])) {
			$metadata['views'] = get_ranking_viewed_item($item_id);
		}

		// xoonips_ranking_downloaded_item
		if (isset($metadata['downloaded'])) {
			$metadata['downloaded'] = get_ranking_downloaded_item($item_id);
		}

                // weko_item_name
                if (isset($metadata['weko_item_name'])) {
                        $metadata['weko_item_name'] = get_weko_item_name($this->item_type);
                }

		return true;
	}
	
	public function exportItemFile($item_id, $item_dir, &$metadata)
	{
		global $delimiter;
		$fname = "";
		$file = get_file_info($item_id);
		if (!$file) {
			return 2;
		}
		
		$export_file_dir = $item_dir."/".$item_id;
        if ( !mkdir( $export_file_dir, 0755 ) ) {
			add_log("ERROR can not make directory ".$export_file_dir."\n");
			return false;
		} else {
			foreach ($file as $f) {
				$same_file = false;
				list( $file_id, $file_type_name, $original_file_name, $file_size, $mime_type, $thumbnail_file, $caption) = $f;
				$copy_from = xnpGetUploadFilePath($file_id);
				if ( file_exists( $copy_from ) ) {
					if (empty($caption)) {
						$fname = windows_sanitize($original_file_name);
					} else {
						$fname = windows_sanitize($caption);
					}
					
					$export_file_path = $export_file_dir.'/'.mb_convert_encoding($fname, "UTF-8", "auto");
					
					if (file_exists($export_file_path)) {
						if (is_same_file($copy_from, $export_file_path)){
							$same_file = true;
						}else{
							$fextension = pathinfo($fname, PATHINFO_EXTENSION );
							if (!$fextension) {
								$fbase = $fname;
							} else {
								$fextension = ".".$fextension;
								$fbase = basename($fname, $fextension);
							}						
							
							for ($i = 1; file_exists($export_file_path); $i++) {
								if (!$fextension) {
									$fname = sprintf('%s_%03s', $fbase, $i);
								} else {
									$fname = sprintf('%s_%03s%s', $fbase, $i, $fextension);
								}
								$fname = windows_sanitize($fname);
								$export_file_path = $export_file_dir.'/'.mb_convert_encoding($fname, "UTF-8", "auto");
							}
						}
					}
					
					if (!$same_file && !@copy( $copy_from, $export_file_path)) {
						add_log("Warning can not copy from ".$copy_from."\n");
						return false;
					} else {
						if ($file_type_name == "stimulus_data") {
							if (isset($metadata['file'])) {
								$metadata['file'] = $fname;
							}
							if (isset($metadata['mime_type'])) {
								$metadata['mime_type'] = $mime_type;
							}
						} elseif ($file_type_name == "preview") {
							if (isset($metadata['preview'])) {
								if (empty($metadata['preview'])) {
									$metadata['preview'] = $fname;
								} else {
									$metadata['preview'] .= $delimiter . $fname;
								}	
							}
						} else {
							// 無視
						}
						add_log("file copy `".$copy_from."' -> `" . $export_file_path . "'\n");
					}
				} else {
					add_log("Warning no exists file " . $copy_from . "\n");
				}
			}
		}
		
		return true;
	}
	
	private function getStimulusItemDetail($item_id)
	{
		$stimulus_details = array();
		global $xoopsDB;
		$sql = "SELECT stimulus_type, readme, rights, use_cc, cc_commercial_use, cc_modification FROM " . $xoopsDB->prefix( "xnpstimulus_item_detail" );
		$sql .= " WHERE stimulus_id = " . $item_id;
		$result = $xoopsDB->query( $sql );
		if ( !$result ) {
			add_log("ERROR getStimulusItemDetail false ".$sql."\n");
			return false;
		} else {
			while ( $row = $xoopsDB->fetchArray( $result ) ) {
				if ($row['use_cc'] === "1") {
					$cc_str = "CC : BY";
					if ($row['cc_commercial_use'] === "0") {
						$cc_str .= "-NC";
					}
					if ($row['cc_modification'] === "1") {
						$cc_str .= "-SA";
					} else if ($row['cc_modification'] === "0") {
						$cc_str .= "-ND";
					}
					$row['rights'] = $cc_str;
				}
				$stimulus_details = $row;
			}
		}
        return $stimulus_details;
	}
	
	private function getStimulusDeveloper($item_id)
	{
		$developer = "";
		global $xoopsDB;
		global $delimiter;
		$sql = "SELECT developer FROM " . $xoopsDB->prefix( "xnpstimulus_developer" );
		$sql .= " WHERE stimulus_id = " . $item_id;
		$result = $xoopsDB->query( $sql );
		if ( !$result ) {
			add_log("ERROR getStimulusDeveloper false ".$sql."\n");
			return false;
		} else {
			while ( $row = $xoopsDB->fetchArray( $result ) ) {
				if (empty($developer)) {
					$developer = $row['developer'];
				} else {
					$developer .= $delimiter . $row['developer'];
				}
			}
		}
		return $developer;
	}
}

